#文件名：kNN.py

import numpy as np
from math import sqrt
from collections import Counter

class KNNClassifier:

    def __init__(self, k): #kNN分类器的构造函数
        assert k >= 1, "k must be valid"
        self.k = k
        self._X_train = None
        self._y_train = None

    def fit(self, X_train, y_train): #根据训练数据集X_train和y_train训练kNN分类器
        assert X_train.shape[0] == y_train.shape[0], \
        "The size of X_train must be equal to the size of y_train."
        assert self.k <= X_train.shape[0], \
        "The size of X_train must be at least k."

        self._X_train = X_train
        self._y_train = y_train
        return self

    def predict(self, X_predict): #给定待预测数据集X_predict，返回表示X_predict的结果向量
        assert self._X_train is not None and self._y_train is not None, \
        "Must fit before predict!"
        assert X_predict.shape[1] == self._X_train.shape[1], \
        "The feature number of X_predict must be equal to X_train."

        y_predict = [self._predict(x) for x in X_predict]
        return np.array(y_predict)

    def _predict(self, x): #给定单个待预测数据x，返回x的预测结果值
        assert x.shape[0] == self._X_train.shape[1], \
        "The feature number of x must be equal to X_train."

        #计算待预测数据x到数据集各点的距离，存在distances数组中
        distances = [sqrt(np.sum((x_train - x)**2)) for x_train in self._X_train]
        nearest = np.argsort(distances) #根据索引进行排序
        topK_y = [self._y_train[i] for i in nearest[:self.k]] #找出距离最短的k个点
        votes = Counter(topK_y) #统计距离最短的k个点中每个标记出现的频率

        return votes.most_common(1)[0][0] #返回出现频率最高的标记

    def __repr__(self): #定义显示方法，返回当前对象的k值
        return "kNN(k=%d)" % self.k


