import numpy as np

#将数据 X 和 y 按照test_ratio分割成X_train, X_test, y_train, y_test
def train_test_split(X, y, test_ratio = 0.2, seed = None):
    assert  X.shape[0] == y.shape[0], \
    "The size of X must be equal to the size of y."
    assert 0.0 <= test_ratio <= 1.0, \
    "Test_ration must be valid."

    #在debug时若需要在运行中生成相同的训练集和测试集，则指定随机数种子
    if seed:
        np.random.seed(seed)

    shuffled_indexes = np.random.permutation(len(X)) #将数据集的顺序打乱（打乱索引）

    #确定训练集和数据集的索引范围
    test_size = int(len(X) * test_ratio)
    test_indexes = shuffled_indexes[:test_size]
    train_indexes = shuffled_indexes[test_size:]

    #使用numpy的fancy indexing来构建训练集和测试集矩阵
    X_train = X[train_indexes]
    y_train = y[train_indexes]

    X_test = X[test_indexes]
    y_test = y[test_indexes]

    return X_train, X_test, y_train, y_test